==> dbf2msql-1.01.tar.gz: dbf2msql-1.01/msql2dbf.c <==
/*	utility to read out an mSQL-table, and store it into a DBF-file

	M. Boekhold (boekhold@cindy.et.tudelft.nl) April 1996
*/

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <msql.h>
#include "dbf.h"

int		verbose = 0, upper = 0, lower = 0, create = 0;
long	precision = 6;
char	*host = NULL;
char	*dbase = NULL;
char	*table = NULL;

inline void strtoupper(char *string);
inline void strtolower(char *string);
void usage(void);

inline void strtoupper(char *string) {
        while(*string != '\0') {
                *string++ = toupper(*string);
        }
}

inline void strtolower(char *string) {
        while(*string != '\0') {
                *string++ = tolower(*string);
        }
}

void usage(void) {
	printf("msql2dbf %s\n", VERSION);
	printf("usage:\tmsql2dbf [-h host] [-u | -l] [-v[v]]\n");
	printf("\t\t\t[-p precision] [-q query] -d dbase -t table dbf-file\n");
}

int main(int argc, char **argv) {
	int             SQLsock, i;
	extern int      optind;
	extern char     *optarg;
	char            *query;
	dbhead          *dbh;
	field			*rec;
	m_result		*qres;
	m_row			qrow;
	m_field			*qfield;
	u_long			numfields, numrows, t;

	while ((i = getopt(argc, argv, "lucvq:h:d:t:p:")) != EOF) {
		switch(i) {
			case 'q':
				query = strdup(optarg);
				break;
			case 'v':
				verbose++;
				break;
			case 'c':
				create = 1;
				break;
			case 'l':
				if (upper) {
					usage();
					printf("Can't use -u and -l at the same time!\n");
					exit(1);
				}
				lower = 1;
				break;
			case 'u':
				if (lower) {
					usage();
					printf("Can't use -u and -l at the same time!\n");
					exit(1);
				}
				upper = 1;
				break;
			case 'h':
				host = (char *)strdup(optarg);
				break;
			case 'd':
				dbase = (char *)strdup(optarg);
				break;
			case 't':
				table = (char *)strdup(optarg);
				break;
			case 'p':
				precision = strtol(optarg, (char **)NULL, 10);
				if ((precision == LONG_MIN) || (precision == LONG_MAX)) {
					usage();
					printf("Invalid precision specified!\n");
					exit(1);
				}
				break;
			case ':':
				usage();
				printf("Missing argument!\n");
				exit(1);
			case '?':
				usage();
				printf("Unknown argument: %s\n", argv[0]);
				exit(1);
			default:
				break;
		}
	}

	if (query == NULL) {
		query = (char *)malloc(14+strlen(table));
		sprintf(query, "SELECT * FROM %s", table);
	}

	if ((dbase == NULL) || ((table == NULL) && (query == NULL))) {
		usage();
		printf("Dbase and table *must* be specified!\n");
		exit(1);
	}

	argc -= optind;
	argv = &argv[optind];

	if (argc != 1) {
		usage();
		exit(1);
	}

	if (verbose > 1) {
		printf("Opening %s for writing (previous contents will be lost)\n",
																	argv[0]);
	}

	if ((dbh = dbf_open_new(argv[0], O_WRONLY | O_CREAT | O_TRUNC)) ==
												(dbhead *)DBF_ERROR) {
		fprintf(stderr, "Couldn't open xbase-file for writing: %s\n", argv[0]);
		exit(1);
	}

	if (verbose > 1) {
		printf("Making connectiion with mSQL-server\n");
	}

    if ((SQLsock = msqlConnect(host)) == -1) {
        fprintf(stderr, "Couldn't get a connection with the ");
        fprintf(stderr, "designated host!\n");
        fprintf(stderr, "Detailed report: %s\n", msqlErrMsg);
        close(dbh->db_fd);
        free(dbh);
        exit(1);
    }

    if (verbose > 1) {
        printf("Selecting database\n");
    }

    if ((msqlSelectDB(SQLsock, dbase)) == -1) {
        fprintf(stderr, "Couldn't select database %s.\n", dbase);
        fprintf(stderr, "Detailed report: %s\n", msqlErrMsg);
        close(dbh->db_fd);
        free(dbh);
        msqlClose(SQLsock);
        exit(1);
    }

	if (verbose > 1) {
		printf("Sending query\n");
	}

	if (msqlQuery(SQLsock, query) == -1) {
		fprintf(stderr, "Error sending query.\nDetailed report: %s\n",
				msqlErrMsg);
		if (verbose > 1) {
			fprintf(stderr, "%s\n", query);
		}
		msqlClose(SQLsock);
		close(dbh->db_fd);
		free(dbh);
		exit(1);
	}

    qres            = msqlStoreResult();
    numfields       = msqlNumFields(qres);
	numrows			= msqlNumRows(qres);

	while ((qfield = msqlFetchField(qres))) {
		strtoupper(qfield->name);
		switch (qfield->type) {
			case INT_TYPE:
				dbf_add_field(dbh, qfield->name, 'N', qfield->length, 0);
				if (verbose > 1) {
					printf("Adding field: %s, INT_TYPE, %d\n", qfield->name,
															   qfield->length);
				}
				break;
			case REAL_TYPE:
				dbf_add_field(dbh, qfield->name, 'N', qfield->length,
								(int) precision);
				if (verbose > 1) {
					printf("Adding field: %s, INT_REAL, %d\n", qfield->name,
															   qfield->length);
				}
				break;
			case CHAR_TYPE:
				dbf_add_field(dbh, qfield->name, 'C', qfield->length, 0);
				if (verbose > 1) {
					printf("Adding field: %s, INT_CHAR, %d\n", qfield->name,
															   qfield->length);
				}
				break;
			default:
				break;
		}
	}

	if (verbose > 1) {
		printf("Writing out header and field-descriptions\n");
	}

	dbf_write_head(dbh);
	dbf_put_fields(dbh);

	if (numrows) {
		while ((qrow = msqlFetchRow(qres)) != NULL) {
			if ((rec = dbf_build_record(dbh)) != (field *)DBF_ERROR) {
				for (t = 0; t < numfields; t++) {
					rec[t].db_contents = strdup(qrow[t]);
					if (upper) {
						strtoupper(rec[t].db_contents);
					}
					if (lower) {
						strtolower(rec[t].db_contents);
					}
				}
				dbf_put_record(dbh, rec, dbh->db_records + 1);
				dbf_free_record(dbh, rec);
			}
		}
	}

	if (verbose > 1) {
		printf("Writing out header\n");
	}

	dbf_write_head(dbh);

	if (verbose > 1) {
		printf("Closing up\n");
	}
				
	msqlFreeResult(qres);
	msqlClose(SQLsock);
	close(dbh->db_fd);
	free(dbh);
	exit(0);
}

==> dbf2msql-1.02b.tar.gz: dbf2msql-1.02b/msql2dbf.c <==
/*	utility to read out an mSQL-table, and store it into a DBF-file

	M. Boekhold (boekhold@cindy.et.tudelft.nl) April 1996
*/

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <msql.h>
#include "dbf.h"

int		verbose = 0, upper = 0, lower = 0, create = 0;
long	precision = 6;
char	*host = NULL;
char	*dbase = NULL;
char	*table = NULL;

inline void strtoupper(char *string);
inline void strtolower(char *string);
void usage(void);

inline void strtoupper(char *string) {
        while(*string != '\0') {
                *string = toupper(*string);
				*string++;
        }
}

inline void strtolower(char *string) {
        while(*string != '\0') {
                *string = tolower(*string);
				*string++;
        }
}

void usage(void) {
	printf("msql2dbf %s\n", VERSION);
	printf("usage:\tmsql2dbf [-h host] [-u | -l] [-v[v]]\n");
	printf("\t\t\t[-p precision] [-q query] -d dbase -t table dbf-file\n");
}

int main(int argc, char **argv) {
	int             SQLsock, i;
	extern int      optind;
	extern char     *optarg;
	char            *query = NULL;
	dbhead          *dbh;
	field			*rec;
	m_result		*qres;
	m_row			qrow;
	m_field			*qfield;
	u_long			numfields, numrows, t;

	while ((i = getopt(argc, argv, "lucvq:h:d:t:p:")) != EOF) {
		switch(i) {
			case 'q':
				query = strdup(optarg);
				break;
			case 'v':
				verbose++;
				break;
			case 'c':
				create = 1;
				break;
			case 'l':
				if (upper) {
					usage();
					printf("Can't use -u and -l at the same time!\n");
					exit(1);
				}
				lower = 1;
				break;
			case 'u':
				if (lower) {
					usage();
					printf("Can't use -u and -l at the same time!\n");
					exit(1);
				}
				upper = 1;
				break;
			case 'h':
				host = (char *)strdup(optarg);
				break;
			case 'd':
				dbase = (char *)strdup(optarg);
				break;
			case 't':
				table = (char *)strdup(optarg);
				break;
			case 'p':
				precision = strtol(optarg, (char **)NULL, 10);
				if ((precision == LONG_MIN) || (precision == LONG_MAX)) {
					usage();
					printf("Invalid precision specified!\n");
					exit(1);
				}
				break;
			case ':':
				usage();
				printf("Missing argument!\n");
				exit(1);
			case '?':
				usage();
				printf("Unknown argument: %s\n", argv[0]);
				exit(1);
			default:
				break;
		}
	}

	if (query == NULL) {
		query = (char *)malloc(14+strlen(table));
		sprintf(query, "SELECT * FROM %s", table);
	}

	if ((dbase == NULL) || ((table == NULL) && (query == NULL))) {
		usage();
		printf("Dbase and table *must* be specified!\n");
		exit(1);
	}

	argc -= optind;
	argv = &argv[optind];

	if (argc != 1) {
		usage();
		exit(1);
	}

	if (verbose > 1) {
		printf("Opening %s for writing (previous contents will be lost)\n",
																	argv[0]);
	}

	if ((dbh = dbf_open_new(argv[0], O_WRONLY | O_CREAT | O_TRUNC)) ==
												(dbhead *)DBF_ERROR) {
		fprintf(stderr, "Couldn't open xbase-file for writing: %s\n", argv[0]);
		exit(1);
	}

	if (verbose > 1) {
		printf("Making connection with mSQL-server\n");
	}

    if ((SQLsock = msqlConnect(host)) == -1) {
        fprintf(stderr, "Couldn't get a connection with the ");
        fprintf(stderr, "designated host!\n");
        fprintf(stderr, "Detailed report: %s\n", msqlErrMsg);
        close(dbh->db_fd);
        free(dbh);
        exit(1);
    }

    if (verbose > 1) {
        printf("Selecting database\n");
    }

    if ((msqlSelectDB(SQLsock, dbase)) == -1) {
        fprintf(stderr, "Couldn't select database %s.\n", dbase);
        fprintf(stderr, "Detailed report: %s\n", msqlErrMsg);
        close(dbh->db_fd);
        free(dbh);
        msqlClose(SQLsock);
        exit(1);
    }

	if (verbose > 1) {
		printf("Sending query\n");
	}

	if (msqlQuery(SQLsock, query) == -1) {
		fprintf(stderr, "Error sending query.\nDetailed report: %s\n",
				msqlErrMsg);
		if (verbose > 1) {
			fprintf(stderr, "%s\n", query);
		}
		msqlClose(SQLsock);
		close(dbh->db_fd);
		free(dbh);
		exit(1);
	}

    qres            = msqlStoreResult();
    numfields       = msqlNumFields(qres);
	numrows			= msqlNumRows(qres);

	while ((qfield = msqlFetchField(qres))) {
		strtoupper(qfield->name);
		switch (qfield->type) {
			case INT_TYPE:
				dbf_add_field(dbh, qfield->name, 'N', qfield->length, 0);
				if (verbose > 1) {
					printf("Adding field: %s, INT_TYPE, %d\n", qfield->name,
															   qfield->length);
				}
				break;
			case REAL_TYPE:
				dbf_add_field(dbh, qfield->name, 'N', qfield->length,
								(int) precision);
				if (verbose > 1) {
					printf("Adding field: %s, INT_REAL, %d\n", qfield->name,
															   qfield->length);
				}
				break;
			case CHAR_TYPE:
				dbf_add_field(dbh, qfield->name, 'C', qfield->length, 0);
				if (verbose > 1) {
					printf("Adding field: %s, INT_CHAR, %d\n", qfield->name,
															   qfield->length);
				}
				break;
			default:
				break;
		}
	}

	if (verbose > 1) {
		printf("Writing out header and field-descriptions\n");
	}

	dbf_write_head(dbh);
	dbf_put_fields(dbh);

	if (numrows) {
		while ((qrow = msqlFetchRow(qres)) != NULL) {
			if ((rec = dbf_build_record(dbh)) != (field *)DBF_ERROR) {
				for (t = 0; t < numfields; t++) {
					rec[t].db_contents = strdup(qrow[t]);
					if (upper) {
						strtoupper(rec[t].db_contents);
					}
					if (lower) {
						strtolower(rec[t].db_contents);
					}
				}
				dbf_put_record(dbh, rec, dbh->db_records + 1);
				dbf_free_record(dbh, rec);
			}
		}
	}

	if (verbose > 1) {
		printf("Writing out header\n");
	}

	dbf_write_head(dbh);

	if (verbose > 1) {
		printf("Closing up\n");
	}
				
	msqlFreeResult(qres);
	msqlClose(SQLsock);
	close(dbh->db_fd);
	free(dbh);
	exit(0);
}
